﻿using System;
using System.Drawing;

namespace Game131029
{
	public struct Cell
	{
		private static readonly Bitmap[][] images = new Bitmap[][]
		{
			null,
			null,
			new Bitmap[] { Properties.Resources.Hiyo1, Properties.Resources.Hiyo2, Properties.Resources.Hiyo3 },
			new Bitmap[] { Properties.Resources.Niwa1, Properties.Resources.Niwa2, Properties.Resources.Niwa3 },
			new Bitmap[] { Properties.Resources.Niku, Properties.Resources.Niku2 },
		};
		private static readonly Bitmap eggImage = Properties.Resources.Tama;
		/// <summary>
		/// 0 = None, 1 = Cage, 2 = Chicken, 3 = Hen, 4 = Meat
		/// </summary>
		private int exist;
		private int time;
		private int hunger;
		private int life;
		private int sick;
		private int egg;
		private int value;
		private int pattern;

		private bool IsEmpty
		{
			get
			{
				return this.exist < 2;
			}
		}
		private bool IsChicken
		{
			get
			{
				return this.exist == 2;
			}
		}
		private bool IsHen
		{
			get
			{
				return this.exist == 3;
			}
		}
		private bool IsMeat
		{
			get
			{
				return this.exist == 4;
			}
		}
		private bool IsAlive
		{
			get
			{
				return this.exist == 2 || this.exist == 3;
			}
		}
		private bool IsOvereating
		{
			get
			{
				return this.hunger < -60;
			}
		}
		private bool IsSatiated
		{
			get
			{
				return this.hunger < -24;
			}
		}
		private bool IsHungry
		{
			get
			{
				return this.hunger > 24;
			}
		}
		private bool IsStarved
		{
			get
			{
				return this.hunger > 60;
			}
		}
		private bool IsIllness
		{
			get
			{
				return this.sick > 12;
			}
		}
		private bool IsWeak
		{
			get
			{
				return this.life < 24;
			}
		}
		private bool IsTinyChicken
		{
			get
			{
				return this.value < 20;
			}
		}
		private bool IsTinyHen
		{
			get
			{
				return this.value < 80;
			}
		}
		private bool IsLargeChicken
		{
			get
			{
				return this.value > 60;
			}
		}
		private bool IsLargeHen
		{
			get
			{
				return this.value > 240;
			}
		}
		private bool IsFreshness
		{
			get
			{
				return this.time < 20;
			}
		}
		private bool IsYoung
		{
			get
			{
				return this.time < 120;
			}
		}
		private bool IsOld
		{
			get
			{
				return this.time > 240;
			}
		}

		public void Draw(Graphics g, int left, int top)
		{
			if (this.exist > 0)
			{
				g.DrawRectangle(Pens.White, new Rectangle(left, top, 27, 27));
				g.DrawRectangle(Pens.White, new Rectangle(--left, --top, 29, 29));
				if (this.exist > 1)
				{
					g.DrawImage(images[this.exist][this.pattern], left + 7, top + 7, 16, 16);
					if (this.egg >= 0)
					{
						g.DrawImage(eggImage, left + 11, top + 11, 16, 16); 
					}
				}
			}
			else
			{
				g.DrawRectangle(Pens.Lime, new Rectangle(left, top, 27, 27));
			}
		}

		public void DrawStatus(Graphics g, Font font)
		{
			if (this.IsEmpty)
			{
				return;
			}
			const int left = 0;
			int top = 18;
			g.DrawString(this.IsChicken ? " [ひよこ]" : this.IsHen ? "[にわとり]" : "   [肉]", font, Brushes.Black, left, 2);
			if (this.IsAlive)
			{
				if (this.IsChicken)
				{
					if (this.IsTinyChicken)
					{
						g.DrawString("小さい", font, Brushes.Black, left, top);
						top += 16;
					}
					else if (this.IsLargeChicken)
					{
						g.DrawString("大きい", font, Brushes.Black, left, top);
						top += 16;
					}
				}
				else
				{
					if (this.IsTinyHen)
					{
						g.DrawString("小さい", font, Brushes.Black, left, top);
						top += 16;
					}
					else if (this.IsLargeHen)
					{
						g.DrawString("大きい", font, Brushes.Black, left, top);
						top += 16;
					}
				}
				if (this.IsYoung)
				{
					g.DrawString("若い", font, Brushes.Black, left, top);
					top += 16;
				}
				else if (this.IsOld)
				{
					g.DrawString("老いている", font, Brushes.Red, left, top);
					top += 16;
				}
				if (this.IsIllness)
				{
					g.DrawString("病気", font, Brushes.Red, left, top);
					top += 16;
				}
				if (this.IsWeak)
				{
					g.DrawString("衰弱", font, Brushes.Red, left, top);
					top += 16;
				}
				if (this.IsStarved)
				{
					g.DrawString("飢えている", font, Brushes.Red, left, top);
				}
				else if (this.IsHungry)
				{
					g.DrawString("空腹", font, Brushes.Yellow, left, top);
				}
				else if (this.IsOvereating)
				{
					g.DrawString("食べすぎ", font, Brushes.Red, left, top);
				}
				else if (this.IsSatiated)
				{
					g.DrawString("満腹", font, Brushes.Black, left, top);
				}
			}
			else if (this.IsMeat)
			{
				if (this.IsFreshness)
				{
					g.DrawString("新鮮", font, Brushes.Black, left, top);
				}
			}
		}

		public bool Build()
		{
			if (this.exist > 0)
			{
				return false;
			}
			++this.exist;
			return true;
		}

		public bool Birth()
		{
			if (this.exist != 1)
			{
				return false;
			}
			++this.exist;
			this.time = 0;
			this.hunger = 0;
			this.life = 48;
			this.sick = 0;
			this.egg = -1;
			this.value = 0;
			this.pattern = 0;
			return true;
		}

		public bool Feed()
		{
			if (!this.IsAlive)
			{
				return false;
			}
			this.hunger -= 24;
			return true;
		}

		public bool Kill()
		{
			if (!this.IsHen)
			{
				return false;
			}
			++this.exist;
			this.value += this.life;
			this.time = 0;
			this.hunger = 0;
			this.life = 0;
			this.egg = 0;
			this.pattern = this.value > 160 ? 1 : 0;
			return true;
		}

		public int Sell()
		{
			if (!this.IsMeat)
			{
				return 0;
			}
			this.exist = 1;
			return Math.Max(1, this.value - this.sick - this.time);
		}

		public bool Inject()
		{
			if (!this.IsAlive)
			{
				return false;
			}
			this.life += 4;
			this.sick = 0;
			return true;
		}

		public int TimePasses()
		{
			++this.time;
			if (this.IsAlive)
			{
				if (++this.pattern > 2)
				{
					this.pattern = 0;
				}
				++this.hunger;
				if (this.IsStarved)
				{
					this.life -= 2;
					this.sick += 3;
					this.value -= 3;
				}
				else if (this.IsOvereating)
				{
					--this.life;
					++this.sick;
					this.value += 4;
				}
				else if (this.IsSatiated)
				{
					this.value += 2;
				}
				else if (!this.IsHungry)
				{
					++this.life;
					if (--this.sick < 0)
					{
						this.sick = 0;
					}
					++this.value;
				}
				if (this.IsIllness)
				{
					this.life -= 3;
					this.value -= 4;
				}
				if (this.time > 300)
				{
					this.life -= 2;
					this.value -= 2;
				}
				else if (this.IsOld)
				{
					--this.life;
					--this.value;
				}
				if (this.life <= 0)
				{
					this.exist = 1;
					return -1;
				}
				else if (this.life > 60)
				{
					this.life = 60;
				}
				if (this.IsHen && !this.IsIllness && !this.IsHungry)
				{
					if (++this.egg == 0)
					{
						this.hunger -= 4;
						return 4 + (this.value >> 5);
					}
					else if (this.egg == 4)
					{
						this.egg = -12;
					}
				}
				else if (this.IsChicken && this.value > 80)
				{
					++this.exist;
					this.value = 60;
					this.egg = -16;
				}
			}
			else if (this.IsMeat && this.value < this.time)
			{
				this.exist = 1;
				return -1;
			}
			return 0;
		}

		public void Clear()
		{
			this.exist = 0;
		}
	}
}
