﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.IO.Compression;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX.DirectSound;

namespace Game131029
{
	public partial class Form1 : Form
	{
		private readonly Device dsDevice = new Device();
		private readonly SecondaryBuffer[][] secondaryBuffers = new SecondaryBuffer[6][];
		private readonly int[] secondaryBuffersIndexes = new int[6];
		private readonly RadioButton[] radioButtons;
		private readonly System.Timers.Timer timer = new System.Timers.Timer();
		private readonly Cell[][] cells;
		private readonly List<Point> cageList = new List<Point>();
		private readonly List<Point> livingList = new List<Point>();
		private readonly List<Score> scoreList = new List<Score>();
		private readonly Rectangle screenRect;
		private Form2 helpDialog;
		private Form3 scoreDialog;
		private Form4 inputNameDialog;
		private Form5 versionDialog;
		private int time;
		private int money;
		private int command;
		private bool first = true;
		private bool playing = false;
		private bool pause = false;
		private Point p = new Point(-1, -1);

		public Form1()
		{
			InitializeComponent();
			this.dsDevice.SetCooperativeLevel(this, CooperativeLevel.Normal);
			this.secondaryBuffers[0] = this.GetSecondaryBuffers(Properties.Resources.burst00, 31187);
			this.secondaryBuffers[1] = this.GetSecondaryBuffers(Properties.Resources.power12, 6138);
			this.secondaryBuffers[2] = this.GetSecondaryBuffers(Properties.Resources.power30, 9074);
			this.secondaryBuffers[3] = this.GetSecondaryBuffers(Properties.Resources.swing02, 5470);
			this.secondaryBuffers[4] = this.GetSecondaryBuffers(Properties.Resources.coin06, 119904);
			this.secondaryBuffers[5] = this.GetSecondaryBuffers(Properties.Resources.cursor16, 7943);
			this.radioButtons = new RadioButton[] { this.radioButton1, this.radioButton2, this.radioButton3, this.radioButton4, this.radioButton5, this.radioButton6 };
			this.timer.AutoReset = true;
			this.timer.Interval = 250;
			this.timer.Elapsed += new System.Timers.ElapsedEventHandler(this.OnTimerEvent);
			this.timer.SynchronizingObject = this;
			this.screenRect = new Rectangle(new Point(0, 0), this.unflickerPanel1.ClientSize);
			this.cells = new Cell[12][];
			for (int y = 0; y < 12; ++y)
			{
				this.cells[y] = new Cell[12];
			}
		}

		protected override void OnResize(EventArgs e)
		{
			if (this.playing && !this.checkBox1.Checked)
			{
				if (this.WindowState == FormWindowState.Minimized)
				{
					this.StopTimer();
				}
				else
				{
					this.StartTimer();
				}
			}
			base.OnResize(e);
		}

		public Score[] GetScores()
		{
			return this.scoreList.ToArray();
		}

		public void SetScores(Score[] scores)
		{
			this.scoreList.AddRange(scores);
		}

		private SecondaryBuffer[] GetSecondaryBuffers(Byte[] gzip, int length)
		{
			SecondaryBuffer[] sbs = new SecondaryBuffer[4];
			using (MemoryStream ms = new MemoryStream(gzip))
			{
				using (GZipStream gzs = new GZipStream(ms, CompressionMode.Decompress))
				{
					Byte[] buf = new byte[length];
					MemoryStream ms2 = new MemoryStream(buf);
					ms2.Write(buf, 0, gzs.Read(buf, 0, length));
					for (int i = 0; i < 4; ++i)
					{
						ms2.Position = 0;
						sbs[i] = new SecondaryBuffer(ms2, this.dsDevice);
					}
				}
			}
			return sbs;
		}

		private void PlaySound()
		{
			if (this.toolStripMenuItem7.Checked)
			{
				this.secondaryBuffers[this.command][this.secondaryBuffersIndexes[this.command]].SetCurrentPosition(0);
				this.secondaryBuffers[this.command][this.secondaryBuffersIndexes[this.command]].Play(0, BufferPlayFlags.Default);
				if (++this.secondaryBuffersIndexes[this.command] > 3)
				{
					this.secondaryBuffersIndexes[this.command] = 0;
				}
			}
		}

		private void PlusMoney(int value)
		{
			this.money += value;
			this.toolStripStatusLabel2.Text = "[$" + this.money.ToString() + "]";
		}

		private void MinusMoney(int value)
		{
			this.money -= value;
			this.toolStripStatusLabel2.Text = "[$" + this.money.ToString() + "]";
		}

		private void UpdatedTime()
		{
			this.toolStripStatusLabel3.Text = "[" + (this.time / 240).ToString() + ":" + (this.time % 240 >> 2).ToString("00") + "]";
		}

		private void StartTimer()
		{
			this.flowLayoutPanel1.Enabled = true;
			this.pause = false;
			this.timer.Start();
		}

		private void StopTimer()
		{
			this.timer.Stop();
			this.pause = true;
			this.flowLayoutPanel1.Enabled = false;
		}

		private void EndGame()
		{
			this.checkBox1.Checked = false;
			this.playing = false;
			this.p = new Point(-1, -1);
			this.timer1.Enabled = false;
			this.unflickerPanel1.Enabled = false;
			this.timer.Enabled = false;
			this.checkBox1.Enabled = false;
			this.radioButton1.Checked = true;
			this.flowLayoutPanel1.Enabled = false;
			this.toolStripMenuItem2.Enabled = true;
			this.toolStripMenuItem3.Enabled = false;
		}

		private void OnTimerEvent(object source, System.Timers.ElapsedEventArgs e)
		{
			for (int i = 0, v; i < this.livingList.Count; ++i)
			{
				v = this.cells[this.livingList[i].Y][this.livingList[i].X].TimePasses();
				if (v < 0)
				{
					this.livingList.RemoveAt(i--);
				}
				else if (v > 0)
				{
					this.PlusMoney(v);
					this.toolStripStatusLabel1.Text = "卵を売った。(+$" + v.ToString() + ")";
				}
			}
			if (++this.time > 720)
			{
				this.toolStripStatusLabel1.Text = "ゲームを終了しました。";
				this.EndGame();
				if (this.money > 500)
				{
					for (int i = 0; i < this.scoreList.Count; ++i)
					{
						if (this.scoreList[i].Value < this.money)
						{
							this.inputNameDialog = this.inputNameDialog ?? new Form4();
							this.scoreDialog = this.scoreDialog ?? new Form3();
							this.inputNameDialog.SetScore(this.money);
							this.inputNameDialog.ShowDialog(this);
							this.scoreList.Insert(i, new Score(this.money, this.inputNameDialog.ScoreName));
							if (this.scoreList.Count > 10)
							{
								this.scoreList.RemoveAt(10);
							}
							Program.Record = true;
							this.scoreDialog.SetScores(this.scoreList.ToArray(), i);
							this.scoreDialog.ShowDialog(this);
							this.UpdatedTime();
							return;
						}
					}
					if (this.scoreList.Count < 10)
					{
						this.inputNameDialog = this.inputNameDialog ?? new Form4();
						this.scoreDialog = this.scoreDialog ?? new Form3();
						this.inputNameDialog.SetScore(this.money);
						this.inputNameDialog.ShowDialog(this);
						this.scoreList.Add(new Score(this.money, this.inputNameDialog.ScoreName));
						this.scoreDialog.SetScores(this.scoreList.ToArray(), this.scoreList.Count - 1);
						this.scoreDialog.ShowDialog(this);
						Program.Record = true;
					}
				}
			}
			this.UpdatedTime();
		}

		private void RadioButton_CheckedChanged(object sender, EventArgs e)
		{
			this.command = ((RadioButton)sender).TabIndex;
		}

		private void ToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
		{
			if (this.playing && !this.checkBox1.Checked)
			{
				this.StopTimer();
			}
		}

		private void ToolStripMenuItem_DropDownClosed(object sender, EventArgs e)
		{
			if (this.playing && !this.checkBox1.Checked)
			{
				this.StartTimer();
			}
		}

		private void unflickerPanel1_MouseDown(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				int x = e.X / 32, y = e.Y / 32;
				switch (this.command)
				{
					case 0:
						if (this.money >= 50 && this.cells[y][x].Build())
						{
							this.cageList.Add(new Point(x, y));
							this.MinusMoney(50);
							this.toolStripStatusLabel1.Text = "鳥かごを作った。(-$90)";
							this.PlaySound();
						}
						break;
					case 1:
						if (this.money >= 35 && this.cells[y][x].Birth())
						{
							this.livingList.Add(new Point(x, y));
							this.MinusMoney(35);
							this.toolStripStatusLabel1.Text = "ひよこを買った。(-$60)";
							this.PlaySound();
						}
						break;
					case 2:
						if (this.money >= 3 && this.cells[y][x].Feed())
						{
							this.MinusMoney(3);
							this.toolStripStatusLabel1.Text = "えさを与えた。(-$3)";
							this.PlaySound();
						}
						break;
					case 3:
						if (this.cells[y][x].Kill())
						{
							this.PlaySound();
						}
						break;
					case 4:
						int v = this.cells[y][x].Sell();
						if (v > 0)
						{
							this.PlusMoney(v);
							this.livingList.Remove(new Point(x, y));
							this.toolStripStatusLabel1.Text = "肉を売った。(+$" + v.ToString() + ")";
							this.PlaySound();
						}
						break;
					case 5:
						if (this.money >= 15 && this.cells[y][x].Inject())
						{
							this.MinusMoney(15);
							this.toolStripStatusLabel1.Text = "注射した。(-$15)";
							this.PlaySound();
						}
						break;
				}
			}
			else if (e.Button == MouseButtons.Right)
			{
				if (++this.command > 5)
				{
					this.command = 0;
				}
				this.radioButtons[this.command].Checked = true;
			}
			else if (e.Button == MouseButtons.Middle)
			{
				if (--this.command < 0)
				{
					this.command = 5;
				}
				this.radioButtons[this.command].Checked = true;
			}
		}

		private void unflickerPanel1_Paint(object sender, PaintEventArgs e)
		{
			for (int y = 0, top = 2; y < 12; ++y, top += 32)
			{
				for (int x = 0, left = 2; x < 12; ++x, left += 32)
				{
					this.cells[y][x].Draw(e.Graphics, left, top);
				}
			}
			if (this.screenRect.Contains(this.p))
			{
				e.Graphics.DrawRectangle(Pens.Red, new Rectangle(this.p.X, this.p.Y, 31, 31));
			}
		}

		private void unflickerPanel2_Paint(object sender, PaintEventArgs e)
		{
			e.Graphics.Clear(this.BackColor);
			if (!this.pause && this.screenRect.Contains(this.p))
			{
				this.cells[this.p.Y / 32][this.p.X / 32].DrawStatus(e.Graphics, this.unflickerPanel2.Font);
			}
		}

		private void timer1_Tick(object sender, EventArgs e)
		{
			this.unflickerPanel1.Refresh();
			this.p = this.unflickerPanel1.PointToClient(Cursor.Position);
			this.p.X -= this.p.X % 32;
			this.p.Y -= this.p.Y % 32;
			this.unflickerPanel2.Refresh();
		}

		private void toolStripMenuItem2_Click(object sender, EventArgs e)
		{
			if (this.first)
			{
				this.first = false;
			}
			else
			{
				for (int i = 0; i < this.cageList.Count; ++i)
				{
					this.cells[this.cageList[i].Y][this.cageList[i].X].Clear();
				}
				this.cageList.Clear();
				this.livingList.Clear();
			}
			this.toolStripStatusLabel1.Text = "ゲームを開始しました。";
			this.time = 0;
			this.UpdatedTime();
			this.money = 0;
			this.PlusMoney(500);
			this.radioButton1.Checked = true;
			this.unflickerPanel1.Enabled = true;
			this.toolStripMenuItem2.Enabled = false;
			this.toolStripMenuItem3.Enabled = true;
			this.flowLayoutPanel1.Enabled = true;
			this.checkBox1.Enabled = true;
			this.timer.Enabled = true;
			this.timer1.Enabled = true;
			this.playing = true;
		}

		private void toolStripMenuItem3_Click(object sender, EventArgs e)
		{
			this.toolStripStatusLabel1.Text = "ゲームを中止しました。";
			this.EndGame();
		}

		private void toolStripMenuItem4_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void toolStripMenuItem6_Click(object sender, EventArgs e)
		{
			if (this.scoreList.Count <= 0)
			{
				this.toolStripStatusLabel1.Text = "スコアが存在しません。";
			}
			else
			{
				this.scoreDialog = this.scoreDialog ?? new Form3();
				this.scoreDialog.SetScores(this.scoreList.ToArray(), -1);
				if (this.playing && !this.checkBox1.Checked)
				{
					this.StopTimer();
					this.scoreDialog.ShowDialog(this);
					this.StartTimer();
				}
				else
				{
					this.scoreDialog.ShowDialog(this);
				}
			}
		}

		private void toolStripMenuItem9_Click(object sender, EventArgs e)
		{
			this.helpDialog = this.helpDialog ?? new Form2();
			if (!this.helpDialog.IsExist)
			{
				this.toolStripStatusLabel1.Text = "マニュアルが存在しません。";
				this.toolStripMenuItem9.Enabled = false;
				return;
			}
			if (this.playing && !this.checkBox1.Checked)
			{
				this.StopTimer();
				this.helpDialog.ShowDialog(this);
				this.StartTimer();
			}
			else
			{
				this.helpDialog.ShowDialog(this);
			}
		}

		private void toolStripMenuItem10_Click(object sender, EventArgs e)
		{
			this.versionDialog = this.versionDialog ?? new Form5();
			if (this.playing && !this.checkBox1.Checked)
			{
				this.StopTimer();
				this.versionDialog.ShowDialog(this);
				this.StartTimer();
			}
			else
			{
				this.versionDialog.ShowDialog(this);
			}
		}

		private void checkBox1_CheckedChanged(object sender, EventArgs e)
		{
			if (this.checkBox1.Checked)
			{
				this.StopTimer();
			}
			else
			{
				this.StartTimer();
			}
		}
	}
}
